/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
/**
* \file Cartesian.h
* \brief Provides a cartesian topology of the application. 
* 
* \details Topology means "(x,y,z) -> threas" in 3D case. 
*/
#ifndef CUBE_CARTESIAN_H
#define CUBE_CARTESIAN_H   


#include <iosfwd>


/*
 *----------------------------------------------------------------------------
 *
 * class Cartesian
 *
 *----------------------------------------------------------------------------
 */

#include <vector>
#include <map>

namespace cube {

  class Sysres;

/**
* Defines and provides cartesian ( x, y, z )-> thread topology 
*/
  class Cartesian {

  public:

    Cartesian (long ndims, const std::vector<long>& dimv, const std::vector<bool>& periodv)
      : ndims(ndims), dimv(dimv), periodv(periodv) { }
    
    void def_coords(Sysres* sys, const std::vector<long>& coordv);
                                        
    void writeXML  (std::ostream& out) const;

    long                     get_ndims()   const { return ndims; } ///< Returns a number of dimensions of the topology.
    const std::vector<long>& get_dimv()    const { return dimv; } ///< Returns a vector of number of indecies in every dimension.
    const std::vector<bool>& get_periodv() const { return periodv; } ///< Returns a vector of periodicities in every dimension.
    const std::vector<long>& get_coordv(Sysres* sys) const;
    const std::map<const Sysres*, std::vector<long> >&  get_cart_sys() const { return sys2coordv; } ///< Gets a mapping "resource -> coordinate"

    Cartesian* clone();      /// creates a deep copy of himself.


    bool operator== (const Cartesian& b);
    inline bool operator!= (const Cartesian& b) { return !(*this == b);};

  private:

    unsigned long ndims; ///< A number of dimensions in the topology.
    std::vector<long> dimv; ///< Number of indicies in  everydimension
    std::vector<bool> periodv; ///< Periodicity in  everydimension
    std::map<const Sysres*, std::vector<long> > sys2coordv; ///< Mapping "resource -> coordinate".




  };

  
}

#endif
