/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/**
 * @file  epk_mpicom.h
 * 
 * @brief Communicator, group and window management
 */

#ifndef _EPK_MPICOM_H
#define _EPK_MPICOM_H

/* 
 *-----------------------------------------------------------------------------
 *
 *  EPIK Library (Event Processing Interface Kit)
 *
 *  - MPI communicator management
 *
 *-----------------------------------------------------------------------------
 */

#ifdef __cplusplus
#   define EXTERN extern "C" 
#else
#   define EXTERN extern 
#endif

#include "mpi.h"

#include "elg_types.h"

/** @internal Identifier for exposure epochs */
EXTERN const elg_ui1 epk_exp_epoch; 
/** @internal Identifier for access epochs */
EXTERN const elg_ui1 epk_acc_epoch; 

/** 
 * @brief Initialize communicator management 
 */
EXTERN void     epk_comm_init();
/** 
 * @brief Cleanup communicator management 
 */
EXTERN void     epk_comm_finalize();

/** 
 * Create a bitvector resembling the ranks in an MPI group.
 * 
 * Creates a bitvector where nth bit is set, if rank n is part
 * of the group. The bitvector is stored in a static internal 
 * variable.
 *
 * @param  group MPI group handle
 */
EXTERN void     epk_group_to_bitvector(MPI_Group group);

/** 
 * Create an internal handle for an MPI group.
 * 
 * Creates an internal datastructure to enable tracking of this
 * MPI group handle.
 * 
 * @param  group MPI group handle 
 */
EXTERN void     epk_group_create(MPI_Group group);
/** clear internal handle for an MPI group */
EXTERN void     epk_group_free(MPI_Group group);
/** get internal ID for an MPI group */
EXTERN elg_ui4  epk_group_id(MPI_Group group);
/** get index of an MPI group in the internal datastructures */
EXTERN int      epk_group_search(MPI_Group group);

#ifdef HAS_MPI2_1SIDED
EXTERN int      epk_win_rank_to_pe(int rank, MPI_Win win);
EXTERN elg_ui4  epk_win_id(MPI_Win win);
EXTERN void     epk_win_create(MPI_Win win, MPI_Comm comm);
EXTERN void     epk_win_free(MPI_Win win);
EXTERN void     epk_winacc_start(MPI_Win win, MPI_Group group, elg_ui1 color);
EXTERN void     epk_winacc_end(MPI_Win win, elg_ui1 color);
EXTERN elg_ui4  epk_winacc_get_gid(MPI_Win win, elg_ui1 color);
#endif

EXTERN int      epk_comm_get_cid();
EXTERN void     epk_comm_create(MPI_Comm comm);
EXTERN void     epk_comm_free(MPI_Comm comm);

EXTERN elg_ui4  epk_comm_id(MPI_Comm comm);

EXTERN int      epk_rank_to_pe(int rank, MPI_Comm comm);

EXTERN int      epk_comm_determination; /* whether comm/rank determined for events */

/* MPI communicator |-> EPIK communicator id */
#define EPK_COMM_ID(c) (!epk_comm_determination ? ELG_NO_ID : (((c)==MPI_COMM_WORLD) ? 0 : epk_comm_id(c)))

/* Rank with respect to arbitrary communicator |-> global rank */
#define EPK_RANK_TO_PE(r,c) (!epk_comm_determination ? ELG_NO_ID : (((c)==MPI_COMM_WORLD) ? r : epk_rank_to_pe(r,c)))

#endif
