/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "pomp_lib.h"

#include "epk_pomp.h"
#include "epk_omplock.h"
#include "epk_ompreg.h"

#include "esd_event.h"
#include "esd_run.h"

/*
 *----------------------------------------------------------------
 * EPIK implementation of the C POMP function library
 *----------------------------------------------------------------
 */
   
void POMP_Atomic_enter(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Atomic_exit(struct ompregdescr* r) {
  GUARDED_EXIT(rid);
}

void POMP_Barrier_enter(struct ompregdescr* r) {
  GUARDED_ENTER_2('b', rid, brid);
}

void POMP_Barrier_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT_2('b', rid, brid);
}

void POMP_Flush_enter(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Flush_exit(struct ompregdescr* r) {
  GUARDED_EXIT(rid);
}

void POMP_Critical_begin(struct ompregdescr* r) {
  if ( pomp_tracing ) {
    struct EpkRegDescr* data = (struct EpkRegDescr*)(r->data);
    esd_omp_alock(data->brid);
    esd_enter(data->sbrid);
  }
}

void POMP_Critical_end(struct ompregdescr* r) {
  if ( pomp_tracing ) {
    struct EpkRegDescr* data = (struct EpkRegDescr*)(r->data);
    esd_exit(data->sbrid);
    esd_omp_rlock(data->brid);
  }
}

void POMP_Critical_enter(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Critical_exit(struct ompregdescr* r) {
  GUARDED_EXIT(rid);
}

void POMP_For_enter(struct ompregdescr* r) {
  GUARDED_ENTER_2('p', sbrid, rid);
}

void POMP_For_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT_2('p', sbrid, rid);
}

void POMP_Master_begin(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Master_end(struct ompregdescr* r) {
  GUARDED_EXIT(rid);
}

void POMP_Parallel_begin(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Parallel_end(struct ompregdescr* r) {
  GUARDED_COLL_EXIT(rid);
}

void POMP_Parallel_fork(struct ompregdescr* r) {
  if ( pomp_tracing ) {
    struct EpkRegDescr* data = (struct EpkRegDescr*)(r->data);
    esd_omp_fork(data->rid);
  }
}

void POMP_Parallel_join(struct ompregdescr* r) {
  if ( pomp_tracing ) {
    struct EpkRegDescr* data = (struct EpkRegDescr*)(r->data);
    esd_omp_join(data->rid);
  }
}

void POMP_Section_begin(struct ompregdescr* r) {
  GUARDED_ENTER(sbrid);
}

void POMP_Section_end(struct ompregdescr* r) {
  GUARDED_EXIT(sbrid);
}

void POMP_Sections_enter(struct ompregdescr* r) {
  GUARDED_ENTER_2('p', sbrid, rid);
}

void POMP_Sections_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT_2('p', sbrid, rid);
}

void POMP_Single_begin(struct ompregdescr* r) {
  GUARDED_ENTER(sbrid);
}

void POMP_Single_end(struct ompregdescr* r) {
  GUARDED_EXIT(sbrid);
}

void POMP_Single_enter(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Single_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT(rid);
}

void POMP_Workshare_enter(struct ompregdescr* r) {
  GUARDED_ENTER_2('p', sbrid, rid);
}

void POMP_Workshare_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT_2('p', sbrid, rid);
}

/*
 *----------------------------------------------------------------
 * C Wrapper for OpenMP API
 *----------------------------------------------------------------
 */

void POMP_Init_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    esd_enter(epk_omp_regid[EPK__OMP_INIT_LOCK]);
    omp_init_lock(s);
    epk_lock_init(s);
    esd_exit(epk_omp_regid[EPK__OMP_INIT_LOCK]);
  } else {
    omp_init_lock(s);
    epk_lock_init(s);
  }
}

void POMP_Destroy_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    esd_enter(epk_omp_regid[EPK__OMP_DESTROY_LOCK]);
    omp_destroy_lock(s);
    epk_lock_destroy(s);
    esd_exit(epk_omp_regid[EPK__OMP_DESTROY_LOCK]);
  } else {
    omp_destroy_lock(s);
    epk_lock_destroy(s);
  }
}

void POMP_Set_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    esd_enter(epk_omp_regid[EPK__OMP_SET_LOCK]);
    omp_set_lock(s);
    if ( pomp_tracing > 1 ) esd_omp_alock(epk_lock_id(s));
    esd_exit(epk_omp_regid[EPK__OMP_SET_LOCK]);
  } else {
    omp_set_lock(s);
  }
}

void POMP_Unset_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    esd_enter(epk_omp_regid[EPK__OMP_UNSET_LOCK]);
    omp_unset_lock(s);
    if ( pomp_tracing > 1 ) esd_omp_rlock(epk_lock_id(s));
    esd_exit(epk_omp_regid[EPK__OMP_UNSET_LOCK]);
  } else {
    omp_unset_lock(s);
  }
}

int  POMP_Test_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    int result;

    esd_enter(epk_omp_regid[EPK__OMP_TEST_LOCK]);
    result = omp_test_lock(s);
    if (result) if ( pomp_tracing > 1 ) esd_omp_alock(epk_lock_id(s));
    esd_exit(epk_omp_regid[EPK__OMP_TEST_LOCK]);
    return result;     
  } else {
    return omp_test_lock(s);
  }
}

void POMP_Init_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    esd_enter(epk_omp_regid[EPK__OMP_INIT_NEST_LOCK]);
    omp_init_nest_lock(s);
    epk_lock_init(s);
    esd_exit(epk_omp_regid[EPK__OMP_INIT_NEST_LOCK]);
  } else {
    omp_init_nest_lock(s);
    epk_lock_init(s);
  }
}

void POMP_Destroy_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    esd_enter(epk_omp_regid[EPK__OMP_DESTROY_NEST_LOCK]);
    omp_destroy_nest_lock(s);
    epk_lock_destroy(s);
    esd_exit(epk_omp_regid[EPK__OMP_DESTROY_NEST_LOCK]);
  } else {
    omp_destroy_nest_lock(s);
    epk_lock_destroy(s);
  }
}

void POMP_Set_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    esd_enter(epk_omp_regid[EPK__OMP_SET_NEST_LOCK]);
    omp_set_nest_lock(s);
    if ( pomp_tracing > 1 ) esd_omp_alock(epk_lock_id(s));
    esd_exit(epk_omp_regid[EPK__OMP_SET_NEST_LOCK]);
  } else {
    omp_set_nest_lock(s);
  }
}

void POMP_Unset_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    esd_enter(epk_omp_regid[EPK__OMP_UNSET_NEST_LOCK]);
    omp_unset_nest_lock(s);
    if ( pomp_tracing > 1 ) esd_omp_rlock(epk_lock_id(s));
    esd_exit(epk_omp_regid[EPK__OMP_UNSET_NEST_LOCK]);
  } else {
    omp_unset_nest_lock(s);
  }
}

int  POMP_Test_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    int result;

    esd_enter(epk_omp_regid[EPK__OMP_TEST_NEST_LOCK]);
    result = omp_test_nest_lock(s);
    if (result) if ( pomp_tracing > 1 ) esd_omp_alock(epk_lock_id(s));
    esd_exit(epk_omp_regid[EPK__OMP_TEST_NEST_LOCK]);
    return result;
  } else {
    return omp_test_nest_lock(s);
  }
}
