/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#include "MpiComm.h"
#include "Process.h"

using namespace std;
using namespace pearl;


/*
 *---------------------------------------------------------------------------
 *
 * class MpiComm
 *
 *---------------------------------------------------------------------------
 */

//--- Constructors & destructor ---------------------------------------------

/**
 * Creates a new instance and initializes the data with the given values @a
 * id and @a group, respectively.
 *
 * @param id        Communicator identifier
 * @param processes List of processes defining the communicator
 */
MpiComm::MpiComm(ident_t id, const process_group& processes)
  : MpiGroup(id, processes),
    m_communicator(MPI_COMM_NULL)
{
  // Special handling of COMM_SELF
  if (1 == processes.size()) {
    int rank;
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    if (static_cast<ident_t>(rank) == processes[0]->get_id())
      MPI_Comm_dup(MPI_COMM_SELF, &m_communicator);

    return;
  }

  // Special handling of COMM_WORLD
  int size;
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  if (static_cast<uint32_t>(size) == processes.size()) {
    MPI_Comm_dup(MPI_COMM_WORLD, &m_communicator);

    return;
  }

  // Create communicator
  MPI_Comm_create(MPI_COMM_WORLD, get_group(), &m_communicator);
}


/**
 * Destructor. Destroys the instance.
 */
MpiComm::~MpiComm()
{
  // Free resources
  if (m_communicator != MPI_COMM_NULL)
    MPI_Comm_free(&m_communicator);
}


//--- Get communicator information ------------------------------------------

/**
 * Returns a "real" MPI communicator resembling the one used in the target
 * application.
 *
 * @return MPI communicator
 */
MPI_Comm MpiComm::get_comm() const
{
  return m_communicator;
}
