/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#include "MpiGroup.h"
#include "Process.h"

using namespace std;
using namespace pearl;


/*
 *---------------------------------------------------------------------------
 *
 * class MpiGroup
 *
 *---------------------------------------------------------------------------
 */

//--- Constructors & destructor ---------------------------------------------

/**
 * Creates a new instance and initializes the data with the given values @a
 * id and @a group, respectively.
 *
 * @param id        Group identifier
 * @param processes List of processes defining the group
 */
MpiGroup::MpiGroup(ident_t id, const process_group& processes)
  : Group(id, processes), m_group(MPI_GROUP_NULL)
{
  /// @note lazy initialization of m_group in get-method.
}


/**
 * Destructor. Destroys the instance.
 */
MpiGroup::~MpiGroup()
{
  // Free resources
  if (m_group != MPI_GROUP_NULL)
    MPI_Group_free(&m_group);
}


//--- Get group information ------------------------------------------

/**
 * Returns a "real" MPI group resembling the one used in the target
 * application.
 *
 * @return MPI group
 */
MPI_Group MpiGroup::get_group() 
{
  // Lazy generation of MPI group at first access.
  if (m_group == MPI_GROUP_NULL)
  {
    uint32_t count = num_processes();

    // Create rank list
    int* ranks = new int[count];
    for (uint32_t i = 0; i < count; ++i)
      ranks[i] = get_process(i)->get_id();

    // Retrieve world group
    MPI_Group world_group;
    MPI_Comm_group(MPI_COMM_WORLD, &world_group);

    // Create local group
    MPI_Group_incl(world_group, count, ranks, &m_group);

    // Free resources
    MPI_Group_free(&world_group);
    delete[] ranks;
  }

  return m_group;
}
