/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#include <iostream>

#include "Buffer.h"
#include "GlobalDefs.h"
#include "MpiWindow.h"
#include "Location.h"
#include "MpiRmaLock_rep.h"

using namespace std;
using namespace pearl;


/*
 *---------------------------------------------------------------------------
 *
 * class MpiRmaLock_rep
 *
 *---------------------------------------------------------------------------
 */

//--- Event type information ------------------------------------------------

bool MpiRmaLock_rep::is_typeof(event_t type) const
{
  return type == MPI_RMA_LOCK || Event_rep::is_typeof(type);
}


//--- Access event data -----------------------------------------------------

MpiWindow* MpiRmaLock_rep::get_window() const
{
  return m_window;
}

Location* MpiRmaLock_rep::get_location() const
{
  return m_location;
}

bool MpiRmaLock_rep::is_exclusive() const
{
  return m_exclusive;
}


//--- Constructors & destructor (protected) ---------------------------------

MpiRmaLock_rep::MpiRmaLock_rep(timestamp_t timestamp,
                               MpiWindow*  window,
                               Location*   location,
                               bool        exclusive)
  : Event_rep(timestamp),
    m_window(window),
    m_location(location),
    m_exclusive(exclusive)
{
}


MpiRmaLock_rep::MpiRmaLock_rep(const GlobalDefs& defs, Buffer& buffer)
  : Event_rep(defs, buffer)
{
  m_window    = dynamic_cast<MpiWindow*>(defs.get_window(buffer.get_id()));
  m_location  = defs.get_location(buffer.get_id());
  m_exclusive = buffer.get_uint8();
}


//--- Serialize event data (protected) --------------------------------------

void MpiRmaLock_rep::pack(Buffer& buffer) const
{
  Event_rep::pack(buffer);

  buffer.put_id(m_window->get_id());
  buffer.put_id(m_location->get_id());
  buffer.put_uint8(m_exclusive ? 1 : 0);
}


//--- Generate human-readable output of event data (protected) --------------

ostream& MpiRmaLock_rep::output(ostream& stream) const
{
  Event_rep::output(stream);

  return stream << "  win       = " << *m_window << endl
                << "  location  = " <<  m_location << endl
                << "  exclusive = " << m_exclusive << endl;
}
