/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef PEARL_MPIRMALOCK_REP_H
#define PEARL_MPIRMALOCK_REP_H


#include "Event_rep.h"


/*-------------------------------------------------------------------------*/
/**
 * @internal
 *
 * @file    MpiRmaLock_rep.h
 * @ingroup PEARL_mpi
 * @brief   Declaration of the class MpiRmaLock_rep.
 *
 * This header file provides the declaration of the class MpiRmaLock_rep.
 */
/*-------------------------------------------------------------------------*/


namespace pearl
{

/*-------------------------------------------------------------------------*/
/**
 * @internal
 *
 * @class   MpiRmaLock_rep
 * @ingroup PEARL_mpi
 * @brief   Class for MPI RMA lock events
 */
/*-------------------------------------------------------------------------*/

class PEARL_NOPAD_ATTR MpiRmaLock_rep : public Event_rep
{
  public:
    /// @name Constructors & destructor
    /// @{

    MpiRmaLock_rep(timestamp_t timestamp, 
                   MpiWindow* window, 
                   Location* location, 
                   bool exclusive);
    MpiRmaLock_rep(const GlobalDefs& defs, Buffer& buffer);

    /// @}
    /// @name Event type information
    /// @{

    virtual bool is_typeof(event_t type) const;

    /// @}
    /// @name Access event data
    /// @{

    virtual MpiWindow* get_window() const;
    virtual Location*  get_location() const;
    virtual bool       is_exclusive() const;

    /// @}


  protected:
    /// @name Serialize event data
    /// @{

    virtual void pack(Buffer& buffer) const;

    /// @}
    /// @name Generate human-readable output of event data
    /// @{

    virtual std::ostream& output(std::ostream& stream) const;

    /// @}


  private:
    /// MPI window locked
    MpiWindow* m_window;

    /// Lock location
    Location* m_location;

    /// Exclusive lock flag
    bool m_exclusive;
};


}   /* namespace pearl */


#endif   /* !PEARL_MPIRMALOCK_REP_H */
