/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#include <iostream>

#include "Buffer.h"
#include "GlobalDefs.h"
#include "Location.h"
#include "MpiWindow.h"
#include "MpiRmaUnlock_rep.h"

using namespace std;
using namespace pearl;


/*
 *---------------------------------------------------------------------------
 *
 * class MpiRmaUnlock_rep
 *
 *---------------------------------------------------------------------------
 */

//--- Event type information ------------------------------------------------

bool MpiRmaUnlock_rep::is_typeof(event_t type) const
{
  return type == MPI_RMA_UNLOCK || Event_rep::is_typeof(type);
}


//--- Access event data -----------------------------------------------------

MpiWindow* MpiRmaUnlock_rep::get_window() const
{
  return m_window;
}

Location* MpiRmaUnlock_rep::get_location() const
{
  return m_location;
}

//--- Constructors & destructor (protected) ---------------------------------

MpiRmaUnlock_rep::MpiRmaUnlock_rep(timestamp_t timestamp,
                               MpiWindow*  window,
                               Location*   location)
  : Event_rep(timestamp),
    m_window(window),
    m_location(location)
{
}


MpiRmaUnlock_rep::MpiRmaUnlock_rep(const GlobalDefs& defs, Buffer& buffer)
  : Event_rep(defs, buffer)
{
  m_window    = dynamic_cast<MpiWindow*>(defs.get_window(buffer.get_id()));
  m_location  = defs.get_location(buffer.get_id());
}


//--- Serialize event data (protected) --------------------------------------

void MpiRmaUnlock_rep::pack(Buffer& buffer) const
{
  Event_rep::pack(buffer);

  buffer.put_id(m_window->get_id());
  buffer.put_id(m_location->get_id());
}


//--- Generate human-readable output of event data (protected) --------------

ostream& MpiRmaUnlock_rep::output(ostream& stream) const
{
  Event_rep::output(stream);

  return stream << "  win       = " << *m_window << endl
                << "  location  = " <<  m_location << endl;
}
