/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2010                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#include <cstddef>
#include <iomanip>
#include <iostream>

#include "Buffer.h"
#include "Error.h"
#include "Event_rep.h"

using namespace std;
using namespace pearl;


/*
 *---------------------------------------------------------------------------
 *
 * class Event_rep
 *
 *---------------------------------------------------------------------------
 */

//--- Event type information ------------------------------------------------

event_t Event_rep::get_type() const
{
  throw FatalError("EventRep::get_type() -- "
                   "Oops! This method should not be called.");
}


bool Event_rep::is_typeof(event_t type) const
{
  return type == ANY;
}


//--- Access event data -----------------------------------------------------

pearl::timestamp_t Event_rep::get_time() const
{
  return m_timestamp;
}


Region* Event_rep::get_region() const
{
  throw FatalError("EventRep::get_region() -- "
                   "Oops! This method should not be called.");
}


Callsite* Event_rep::get_callsite() const
{
  throw FatalError("EventRep::get_callsite() -- "
                   "Oops! This method should not be called.");
}


MpiComm* Event_rep::get_comm() const
{
  throw FatalError("EventRep::get_comm() -- "
                   "Oops! This method should not be called.");
}


Location* Event_rep::get_root() const
{
  throw FatalError("EventRep::get_root() -- "
                   "Oops! This method should not be called.");
}


uint32_t Event_rep::get_sent() const
{
  throw FatalError("EventRep::get_sent() -- "
                   "Oops! This method should not be called.");
}


uint32_t Event_rep::get_received() const
{
  throw FatalError("EventRep::get_received() -- "
                   "Oops! This method should not be called.");
}


Location* Event_rep::get_source() const
{
  throw FatalError("EventRep::get_source() -- "
                   "Oops! This method should not be called.");
}


Location* Event_rep::get_dest() const
{
  throw FatalError("EventRep::get_dest() -- "
                   "Oops! This method should not be called.");
}


uint32_t Event_rep::get_tag() const
{
  throw FatalError("EventRep::get_tag() -- "
                   "Oops! This method should not be called.");
}


uint32_t Event_rep::get_lockid() const
{
  throw FatalError("EventRep::get_lockid() -- "
                   "Oops! This method should not be called.");
}


MpiWindow* Event_rep::get_window() const
{
  throw FatalError("EventRep::get_window() -- "
                   "Oops! This method should not be called.");
}


Location* Event_rep::get_remote() const
{
  throw FatalError("EventRep::get_remote() -- "
                   "Oops! This method should not be called.");
}


uint32_t Event_rep::get_rma_id() const
{
  throw FatalError("EventRep::get_rma_id() -- "
                   "Oops! This method should not be called.");
}


MpiGroup* Event_rep::get_group() const
{
  throw FatalError("EventRep::get_group() -- "
                   "Oops! This method should not be called.");
}


bool Event_rep::is_sync() const
{
  throw FatalError("EventRep::is_sync() -- "
                   "Oops! This method should not be called.");
}


bool Event_rep::is_exclusive() const
{
  throw FatalError("EventRep::is_exclusive() -- "
                   "Oops! This method should not be called.");
}


/*uint32_t Event_rep::get_lockid() const
{
  throw FatalError("EventRep::get_lockid() -- "
                   "Oops! This method should not be called.");

  return 0;
}*/


#ifdef PEARL_ENABLE_METRICS
uint64_t Event_rep::get_metric(int index) const
{
  throw FatalError("EventRep::get_metric(int) -- "
                   "Oops! This method should not be called.");
}
#endif


//--- Modify event data -----------------------------------------------------

void Event_rep::set_time(timestamp_t timestamp)
{
  m_timestamp = timestamp;
}


//--- Constructors & destructor (protected) ---------------------------------

Event_rep::Event_rep(pearl::timestamp_t timestamp)
  : m_timestamp(timestamp)
{
}


Event_rep::Event_rep(const GlobalDefs& defs, Buffer& buffer)
{
  m_timestamp = buffer.get_time();
}


//--- Serialize event data (protected) --------------------------------------

void Event_rep::pack(Buffer& buffer) const
{
  buffer.put_time(m_timestamp);
}


//--- Generate human-readable output of event data (protected) --------------

ostream& Event_rep::output(ostream& stream) const
{
  return stream << "  time   = " << fixed << m_timestamp << endl;
}
